unit HDSFuncs;
//Common Functions and classes for TWavBitBtn, TMMButton and TMultiLed
//This unit define types, classes, procedures and function that
//could be used  in other components
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DsgnIntf, MMSystem;

type
  TMargin = 3..20;

  TPercent = 0..100; //Percent of both led's shadow and led's Highlight

  TPlayType = (ptNone, ptAsync, ptSync, ptPurge, ptLoop);


  TWaveName = class(TStringProperty)//This class used a open dialog
  public                            //with the filter *.Wav
    { Public Declarations }
     procedure Edit; override;
     function GetAttributes: TPropertyAttributes; override;
  end;

  TWaveFile = class(TPersistent)
  private
     FFileName: TFileName;//Necessary for playing a Wave File
     FPlayType: TPlayType;

  public
     constructor Create;
     procedure Assign(Source: TPersistent); override;
     procedure AssignTo(Dest: TPersistent); override;

  published
     property FileName: TFileName read FFileName write FFileName;
     property PlayType: TPlayType read FPlayType write FPlayType;
  end;

  TWaveFiles = class(TPersistent) //This Class is used for playing the wave
  private                         //files of the control
     FClick: TWaveFile;
     FMouseEnter: TWaveFile;
     FMouseExit: TWaveFile;
     FUseDefault: boolean;
     FControl: TControl;
     FOnChange: TNotifyEvent;
     procedure SetWaveFile(Index: integer; NewWaveFile: TWaveFile);

 public
     constructor Create(AControl: TControl);
     destructor Destroy; override;
     procedure Assign(Source: TPersistent); override;
     procedure AssignTo(Dest: TPersistent); override;
     function CanPlay: Boolean;
     function Play(WaveFile: TWaveFile): boolean;
     function Stop: boolean;
     property Control: TControl read FControl;
     property OnChange: TNotifyEvent read FOnChange write FOnChange;

  published
     property Click: TWaveFile index 0 read FClick write SetWaveFile;
     property MouseEnter: TWaveFile index 1 read FMouseEnter write SetWaveFile;
     property MouseExit: TWaveFile index 2 read FMouseExit write SetWaveFile;
     property UseDefault: boolean read FUseDefault write FUseDefault
                                  default False;
  end;

  TMargins = class(TPersistent)
  private
     FTop: TMargin;
     FBottom: TMargin;
     FLeft: TMargin;
     FRight: TMargin;
     FControl: TControl;
     FOnChange: TNotifyEvent;
     procedure SetTop(const Value: TMargin);
     procedure SetBottom(const Value: TMargin);
     procedure SetLeft(const Value: TMargin);
     procedure SetRight(const Value: TMargin);

  public
     constructor Create(AControl: TControl);
     procedure Assign(Source: TPersistent); override;
     procedure AssignTo(Dest: TPersistent); override;
     property Control: TControl read FControl;
     property OnChange: TNotifyEvent read FOnChange write FOnChange;

  published
     property Top: TMargin read  FTop write SetTop;
     property Bottom: TMargin read  FBottom write SetBottom;
     property Left: TMargin read  FLeft write SetLeft;
     property Right: TMargin read  FRight write SetRight;
  end;

function ChangeBrightness(Color:TColor;Percent:longint):TColor;
function IsAccellerator(VK: Word; const AString: string): boolean;
procedure MoveRect(var Rect: TRect; SpaceX, SpaceY: integer);
function PlayWave(WaveFile: TWaveFile; UseDefault: boolean): boolean;
function StopWave: boolean;
function IsUsingAccellerator(const Text: string): boolean;
function NumberOfAmpersands(const Text: string): integer;

procedure Register;

implementation

function ChangeBrightness(Color:TColor;Percent:longint):TColor;
var
   RGBColor,  NewRed, NewGreen, NewBlue: longint;
   Red, Green, Blue: byte;
   Overflow: longint;
begin
  RGBColor:= ColorToRGB(Color);
  Red:= GetRValue(RGBColor);
  NewRed:= Red + (Percent * Red div 100);

  if NewRed > 255 then
  begin
     Overflow:= NewRed - 255;
     NewGreen:= Overflow;
     NewBlue:= Overflow;
  end
  else
  begin
    NewGreen:= 0;
    NewBlue:= 0;
  end;

  Green:= GetGValue(RGBColor);
  NewGreen:= NewGreen + Green + (Percent * Green div 100);

  if NewGreen > 255 then
  begin
     Overflow:= NewGreen - 255;
     NewRed:= NewRed + Overflow;
     NewBlue:= Overflow;
  end;

  Blue:= GetBValue(RGBColor);
  NewBlue:= NewBlue + Blue + (Percent * Blue div 100);

  if NewBlue > 255 then
  begin
     Overflow:= NewBlue - 255;

     if NewGreen <= 255 then
       NewRed:= NewRed + Overflow;
  end;

  if NewRed > 255 then NewRed:= 255;
  if NewGreen > 255 then NewGreen:= 255;
  if NewBlue > 255 then NewBlue:= 255;
  if NewRed < 0 then NewRed:= 0;
  if NewGreen < 0 then NewGreen:= 0;
  if NewBlue < 0 then NewBlue:= 0;

  Result:= NewRed + (NewGreen shl 8) + (NewBlue shl 16);
end;

function IsAccellerator(VK: Word; const AString: string): boolean;
var
  PosChar: integer;
begin
  PosChar:= Pos('&', AString);
  Result:= (PosChar <> 0) and (PosChar < Length(AString)) and
           (Upcase(AString[PosChar + 1]) = Upcase(Char(VK)));
end;

function IsUsingAccellerator(const Text: string): boolean;
var
  NewText: string;
  PosChar: integer;
begin
  Result:= False;
  NewText:= Text;
  PosChar:= Pos('&', NewText);

  while PosChar > 0 do
  begin
    if (Length(NewText) = PosChar) then
    begin
      Result:= True;
      Exit;
    end;

    if NewText[PosChar + 1] <> '&' then
    begin
       Result:= True;
       Exit;
    end;

    NewText:= Copy(NewText, PosChar + 2, Length(NewText) - PosChar + 2);
    PosChar:= Pos('&', NewText);
  end;
end;

function NumberOfAmpersands(const Text: string): integer;
var
  i: integer;
  TextLength: integer;
begin
  Result:= 0;
  i:= 1;
  TextLength:= Length(Text);

  while i <= TextLength do
  begin
    if Text[i] = '&' then
    begin
      Inc(Result);

      if Text[i + 1] = '&' then
        Inc(i);
    end;

    Inc(i);
  end;
end;

procedure MoveRect(var Rect: TRect; SpaceX, SpaceY: integer);
begin
   with Rect do
   begin
      Inc(Top, SpaceY);
      Inc(Left, SpaceX);
      Inc(Right, SpaceX);
      Inc(Bottom, SpaceY);
   end;
end;

function PlayWave(WaveFile: TWaveFile; UseDefault: boolean): boolean;
var
  AFileName: string;
  Flags: longint;
begin
   with WaveFile do
   begin
      if ((Length(FileName) >= 255) or (PlayType = ptNone)) then
      begin
         Result:= False;
         Exit;
      end;

      AFileName:= FileName;

      if UseDefault then
         Flags:= 0
      else
         Flags:= SND_NODEFAULT;

      case PlayType of
         ptAsync: Flags:= Flags or SND_ASYNC;
         ptSync: Flags:= Flags or SND_SYNC;
         ptLoop: Flags:= Flags or SND_ASYNC or SND_LOOP;
         ptPurge:
         begin
            Flags:= SND_PURGE;
            AFileName:= '';
         end;
      end;

      Result:= PlaySound(PChar(AFileName), 0, Flags);
   end;
end;

function StopWave: boolean;
begin
   Result:= PlaySound('', 0, SND_PURGE);
end;

//Implementation of the class TWaveName
procedure TWaveName.Edit;
begin
   with TOpenDialog.Create(Application) do
     try
        Filename:= GetValue;
        Filter:= 'Wave Files (*.wav)|*.wav|' +
                          'All Files (*.*)|*.*|';
       Options:= [ofFileMustExist, ofPathMustExist];

        if Execute then
          SetValue(Filename);

   finally
      Free;
   end;
end;

function TWaveName.GetAttributes : TPropertyAttributes;
begin
   Result := [paDialog];
end;

//Implementation of the class TWaveFile
constructor TWaveFile.Create;
begin
   inherited Create;
   FFileName:= '';
   FPlayType:= ptNone;
end;

procedure TWaveFile.Assign(Source: TPersistent);
begin
   if(Source is TWaveFile) then
      with TWaveFile(Source) do
      begin
         Self.FFileName:= FFileName;
         Self.FPlayType:= FPlayType;
      end
   else
      inherited Assign(Source);
end;

procedure TWaveFile.AssignTo(Dest: TPersistent);
begin
   if(Dest is TWaveFile) then
      with TWaveFile(Dest) do
      begin
         FFileName:= Self.FFileName;
         FPlayType:= Self.FPlayType;
      end
   else
      inherited AssignTo(Dest);
end;

//Implementation of the class TWaveFiles
constructor TWaveFiles.Create(AControl: TControl);
begin
   inherited Create;
   FControl:= AControl;
   FClick:= TWaveFile.Create;
   FMouseEnter:= TWaveFile.Create;
   FMouseExit:= TWaveFile.Create;
   FUseDefault:= False;
end;

destructor TWaveFiles.Destroy;
begin
   FClick.Free;
   FClick:= nil;
   FMouseEnter.Free;
   FMouseEnter:= nil;
   FMouseExit.Free;
   FMouseExit:= nil;
end;

procedure TWaveFiles.Assign(Source: TPersistent);
begin
   if(Source is TWaveFiles) then
      with TWaveFiles(Source) do
      begin
         Self.FClick.Assign(FClick);
         Self.FMouseEnter.Assign(FMouseEnter);
         Self.FMouseExit.Assign(FMouseExit);
         Self.FUseDefault:= FUseDefault;

         if Assigned(Self.FOnChange) then
           Self.FOnChange(Self);
      end
   else
      inherited Assign(Source);
end;

procedure TWaveFiles.AssignTo(Dest: TPersistent);
begin
   if(Dest is TWaveFiles) then
      with TWaveFiles(Dest) do
      begin
         Self.FClick.AssignTo(FClick);
         Self.FMouseEnter.AssignTo(FMouseEnter);
         Self.FMouseExit.AssignTo(FMouseExit);
         FUseDefault:= Self.FUseDefault;

         if Assigned(FOnChange) then
           FOnChange(Dest);
      end
   else
      inherited AssignTo(Dest);
end;

procedure TWaveFiles.SetWaveFile(Index: integer; NewWaveFile: TWaveFile);
begin
   case Index of
      0: FClick.Assign(NewWaveFile);
      1: FMouseEnter.Assign(NewWaveFile);
      2: FMouseExit.Assign(NewWaveFile);
   end;

   if Assigned(FOnChange) then
     FOnChange(Self);
end;

function TWaveFiles.CanPlay: Boolean;
begin
   Result:= WaveOutGetNumDevs > 0;
end;

function TWaveFiles.Play(WaveFile: TWaveFile): boolean;
begin
   Result:= PlayWave(WaveFile, FUseDefault);
end;

function TWaveFiles.Stop: boolean;
begin
   Result:= StopWave;
end;

//Implementacion of the class TMargins
constructor TMargins.Create(AControl: TControl);
begin
   inherited Create;
   FControl:= AControl;
   FTop:= 3;
   FBottom:= 3;
   FLeft:= 3;
   FRight:= 3;
end;

procedure TMargins.Assign(Source: TPersistent);
begin
   if Source is TMargins then
      with TMargins(Source) do
      begin
         Self.FTop:= FTop;
         Self.FBottom:= FBottom;
         Self.FLeft:= FLeft;
         Self.FRight:= FRight;

         if Assigned(Self.FOnChange) then
           Self.FOnChange(Self);
      end
   else
      inherited Assign(Source);
end;

procedure TMargins.AssignTo(Dest: TPersistent);
begin
   if(Dest is TMargins) then
      with TMargins(Dest) do
      begin
         FTop:= Self.FTop;
         FBottom:= Self.FBottom;
         FLeft:= Self.FLeft;
         FRight:= Self.FRight;

         if Assigned(FOnChange)
           then FOnChange(Dest);
       end
    else
       inherited AssignTo(Dest);
end;

procedure TMargins.SetTop(const Value: TMargin);
begin
  if FTop <> Value then
  begin
    FTop:= Value;

    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TMargins.SetBottom(const Value: TMargin);
begin
  if FBottom <> Value then
  begin
    FBottom:= Value;

    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TMargins.SetLeft(const Value: TMargin);
begin
  if FLeft <> Value then
  begin
    FLeft:= Value;

    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TMargins.SetRight(const Value: TMargin);
begin
  if FRight <> Value then
  begin
    FRight:= Value;

    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure Register;
begin
   RegisterPropertyEditor(TypeInfo(TFileName), TWaveFile, '', TWaveName);
end;

end.
